#include "TriMesh.h"

#include "Node.h"
#include "Edge.h"
#include "TriFace.h"

TriMesh::~TriMesh()
{

}

void TriMesh::populate(VecNodes& nodes, VecEdges& edges, VecFaces& faces)
{
    mNodes = std::move(nodes);
    mEdges = std::move(edges);
    mFaces = std::move(faces);
}

std::vector<unsigned> TriMesh::getFaceNodeIds() const
{
    unsigned nodes_per_face = 3;
    std::vector<unsigned> ids(nodes_per_face*mFaces.size());

    for(std::size_t idx=0; idx<mFaces.size(); idx++)
    {
        const auto nodeIds = mFaces[idx]->getNodeIds();
        for(std::size_t jdx=0; jdx<nodes_per_face; jdx++)
        {
            ids[nodes_per_face*idx + jdx] = nodeIds[jdx];
        }
    }
    return ids;
}

std::vector<std::vector<double> > TriMesh::getFaceVectorAttributes(const std::string& tag)
{
    std::vector<std::vector<double> > attribs(mFaces.size());
    for(std::size_t idx=0; idx<mFaces.size(); idx++)
    {
        attribs[idx] = {mFaces[idx]->getVectorAttribute(tag)};
    }
    return attribs;
}

void TriMesh::addConstantFaceVectorAttribute(const std::string& tag, const std::vector<double>& values)
{
    for (const auto& face : mFaces)
    {
        face->addVectorAttribute(tag, values);
    }
}
