#pragma once

#include "AbstractMesh.h"

#include <string>
#include <unordered_map>

class Edge;
class TriFace;

using EdgePtr = std::unique_ptr<Edge>;
using TriFacePtr = std::unique_ptr<TriFace>;

using VecEdges = std::vector<EdgePtr>;
using VecFaces = std::vector<TriFacePtr>;

class TriMesh : public AbstractMesh
{
public:
    TriMesh() = default;

    ~TriMesh();

    void populate(VecNodes& nodes, VecEdges& edges, VecFaces& faces);

    std::vector<unsigned> getFaceNodeIds() const;

    void addConstantFaceVectorAttribute(const std::string& tag, const std::vector<double>& values);

    std::vector<std::vector<double> > getFaceVectorAttributes(const std::string& tag);

    MeshType getType() const
    {
        return MeshType::TRI;
    }

private:
    VecEdges mEdges;
    VecFaces mFaces;
};
